"""
对话类
1. 获取机器人回复
2. 获取关键字机器人回复
3. 获取问答机器人回复
4. 获取任务机器人回复
5. 同步发给用户的消息
6. 接收用户发的消息
7. 查询历史消息
8. 给用户发消息
9. 获取用户输入联想
"""
from typing import List

from wulaisdk.response import BaseModel

from wulaisdk.response.bot import Bot, Response
from wulaisdk.response.msg_body import MsgBody


class SenderInfo(BaseModel):
    """
    消息发出者信息，可以是人工或机器人
    """
    avatar_url: str
    nickname: str
    real_name: str

    def __init__(self, avatar_url: str, nickname: str, real_name: str) -> None:
        self.avatar_url = avatar_url
        self.nickname = nickname
        self.real_name = real_name


class UserInfo(BaseModel):
    """
    用户信息
    """
    avatar_url: str
    nickname: str

    def __init__(self, avatar_url: str, nickname: str) -> None:
        self.avatar_url = avatar_url
        self.nickname = nickname


class Msg(BaseModel):
    """
    返回的单条消息
    """
    direction: str
    sender_info: SenderInfo
    msg_type: str
    extra: str
    msg_id: str
    msg_ts: str
    user_info: UserInfo
    msg_body: MsgBody

    def __init__(self, direction: str, sender_info: SenderInfo, msg_type: str, extra: str, msg_id: str, msg_ts: str, user_info: UserInfo, msg_body: MsgBody) -> None:
        self.direction = direction
        if sender_info:
            self.sender_info = SenderInfo.from_dict(sender_info)
        else:
            self.sender_info = SenderInfo("", "", "")
        self.msg_type = msg_type
        self.extra = extra
        self.msg_id = msg_id
        self.msg_ts = msg_ts
        if user_info:
            self.user_info = UserInfo.from_dict(user_info)
        else:
            self.user_info = UserInfo("", "")
        self.msg_body = MsgBody.from_dict(msg_body)


class Keyword(BaseModel):
    """
    关键字机器人
    """
    keyword_id: int
    keyword: str

    def __init__(self, keyword_id: int, keyword: str) -> None:
        self.keyword_id = keyword_id
        self.keyword = keyword


class KeywordSuggestedResponse(BaseModel):
    """
    单条关键字机器人应答回复
    """
    is_send: bool
    score: int
    response: List[Response]
    keyword: Keyword
    quick_reply: List[str]

    def __init__(self, is_send: bool, score: int, response: List[Response], keyword: Keyword, quick_reply: List[str]) -> None:
        self.is_send = is_send
        self.score = score
        self.response = [Response.from_dict(r) for r in response]
        self.keyword = Keyword.from_dict(keyword)
        self.quick_reply = quick_reply


class QA(BaseModel):
    """
    问答机器人
    """
    knowledge_id: int
    standard_question: str
    question: str

    def __init__(self, knowledge_id: int, standard_question: str, question: str) -> None:
        self.knowledge_id = knowledge_id
        self.standard_question = standard_question
        self.question = question


class QASuggestedResponse(BaseModel):
    """
    单条问答机器人应答回复
    """
    qa: QA
    is_send: bool
    score: int
    response: List[Response]
    quick_reply: List[str]

    def __init__(self, qa: QA, is_send: bool, score: int, response: List[Response], quick_reply: List[str]) -> None:
        self.qa = QA.from_dict(qa)
        self.is_send = is_send
        self.score = score
        self.response = [Response.from_dict(r) for r in response]
        self.quick_reply = quick_reply


class Entity(BaseModel):
    """
    抽取的实体列表
    """
    idx_end: int
    name: str
    idx_start: int
    value: str
    seg_value: str
    type: str
    desc: str

    def __init__(self, idx_end: int, name: str, idx_start: int, value: str, seg_value: str, type: str, desc: str) -> None:
        self.idx_end = idx_end
        self.name = name
        self.idx_start = idx_start
        self.value = value
        self.seg_value = seg_value
        self.type = type
        self.desc = desc


class Task(BaseModel):
    """
    任务机器人
    """
    block_type: str
    block_id: int
    task_id: int
    block_name: str
    entities: List[Entity]
    task_name: str
    robot_id: int

    def __init__(self, block_type: str, block_id: int, task_id: int, block_name: str, entities: List[Entity],
                 task_name: str, robot_id: int) -> None:
        self.block_type = block_type
        self.block_id = block_id
        self.task_id = task_id
        self.block_name = block_name
        self.entities = [Entity.from_dict(entity) for entity in entities]
        self.task_name = task_name
        self.robot_id = robot_id


class TaskSuggestedResponse(BaseModel):
    """
    单条任务机器人应答回复
    """
    score: int
    is_send: bool
    task: Task
    response: List[Response]
    quick_reply: List[str]

    def __init__(self, score: int, is_send: bool, task: Task, response: List[Response], quick_reply: List[str]) -> None:
        self.score = score
        self.is_send = is_send
        self.task = Task.from_dict(task)
        self.response = [Response.from_dict(r) for r in response]
        self.quick_reply = quick_reply


class SuggestedResponse(BaseModel):
    """
    单条机器人应答回复
    """
    is_send: bool
    bot: Bot
    source: str
    score: int
    response: List[Response]
    quick_reply: List[str]

    def __init__(self, is_send: bool, bot: Bot, source: str, score: int, response: List[Response], quick_reply: List[str]) -> None:
        self.is_send = is_send
        self.bot = Bot.from_dict(bot)
        self.source = source
        self.score = score
        self.response = [Response.from_dict(r) for r in response]
        self.quick_reply = quick_reply


class BotResponse(BaseModel):
    """
    获取机器人回复
    """
    is_dispatch: bool
    suggested_response: List[SuggestedResponse]
    msg_id: str
    extra: str

    def __init__(self, is_dispatch: bool, suggested_response: List[SuggestedResponse], msg_id: str, extra: str) -> None:
        self.is_dispatch = is_dispatch
        self.suggested_response = [SuggestedResponse.from_dict(sr) for sr in suggested_response]
        self.msg_id = msg_id
        self.extra = extra


class KeywordBotResponse(BaseModel):
    """
    获取关键字机器人回复
    """
    is_dispatch: bool
    msg_id: str
    keyword_suggested_response: List[KeywordSuggestedResponse]
    extra: str

    def __init__(self, is_dispatch: bool, msg_id: str, keyword_suggested_response: List[KeywordSuggestedResponse], extra: str) -> None:
        self.is_dispatch = is_dispatch
        self.msg_id = msg_id
        self.keyword_suggested_response = [KeywordSuggestedResponse.from_dict(ksr) for ksr in keyword_suggested_response]
        self.extra = extra


class QABotResponse(BaseModel):
    """
    获取问答机器人回复
    """
    is_dispatch: bool
    msg_id: str
    qa_suggested_response: List[QASuggestedResponse]
    extra: str

    def __init__(self, is_dispatch: bool, msg_id: str, qa_suggested_response: List[QASuggestedResponse], extra: str) -> None:
        self.is_dispatch = is_dispatch
        self.msg_id = msg_id
        self.qa_suggested_response = [QASuggestedResponse.from_dict(qsr) for qsr in qa_suggested_response]
        self.extra = extra


class TaskBotResponse(BaseModel):
    """
    获取任务机器人回复
    """
    is_dispatch: bool
    msg_id: str
    task_suggested_response: List[TaskSuggestedResponse]
    extra: str

    def __init__(self, is_dispatch: bool, msg_id: str, task_suggested_response: List[TaskSuggestedResponse], extra: str) -> None:
        self.is_dispatch = is_dispatch
        self.msg_id = msg_id
        self.task_suggested_response = [TaskSuggestedResponse.from_dict(tsr) for tsr in task_suggested_response]
        self.extra = extra


class SyncMessage(BaseModel):
    """
    同步发给用户的消息
    """
    msg_id: str

    def __init__(self, msg_id: str) -> None:
        self.msg_id = msg_id


class ReceiveMessage(BaseModel):
    """
    接收用户发的消息
    """
    msg_id: str

    def __init__(self, msg_id: str) -> None:
        self.msg_id = msg_id


class HistoryMessage(BaseModel):
    """
    查询历史消息
    """
    msg: List[Msg]
    has_more: bool

    def __init__(self, msg: List[Msg], has_more: bool) -> None:
        self.msg = [Msg.from_dict(m) for m in msg]
        self.has_more = has_more


class SendMessage(BaseModel):
    """
    给用户发消息
    """
    msg_id: str

    def __init__(self, msg_id: str) -> None:
        self.msg_id = msg_id


class UserSuggestion(BaseModel):
    """
    输入联想内容。在相同对话类型中，如有多条联想内容，按照置信度从高到低排序；在不同对话类型中，如有任务对话的联想，则不返回问答对话的联想内容。
    """
    suggestion: str

    def __init__(self, suggestion: str) -> None:
        self.suggestion = suggestion


class GetUserSuggestion(BaseModel):
    """
    获取用户输入联想
    """
    user_suggestions: List[UserSuggestion]

    def __init__(self, user_suggestions: List[UserSuggestion]) -> None:
        self.user_suggestions = [UserSuggestion.from_dict(user_suggestion) for user_suggestion in user_suggestions]





"""
词库管理类
1. 查询全部实体概要
2. 查询专有词汇列表
3. 创建专有词汇
4. 更新专有词汇
5. 删除专有词汇
6. 查询一个实体详情
7. 创建枚举实体
8. 创建枚举实体值
9. 删除枚举实体值
10. 创建意图实体
11. 创建意图实体值相似说法
12. 删除意图实体值相似说法
13. 删除实体
"""

from typing import List
from wulaisdk.response import BaseModel


class Entity(BaseModel):
    """
    实体
    """
    type: str
    id: int
    name: str

    def __init__(self, type: str, id: int, name: str) -> None:
        self.type = type
        self.id = id
        self.name = name


class DictionaryEntities(BaseModel):
    """
    全部实体概要列表
    """
    entities: List[Entity]

    def __init__(self, entities: List[Entity]) -> None:
        self.entities = [Entity.from_dict(entity) for entity in entities]


class Term(BaseModel):
    """
    专有词汇详情
    """
    id: int
    name: str

    def __init__(self, id: int, name: str) -> None:
        self.id = id
        self.name = name


class TermItem(BaseModel):
    """
    专有词汇
    """
    term: Term
    synonyms: List[str]

    def __init__(self, term: Term, synonyms: List[str]) -> None:
        self.term = Term.from_dict(term)
        self.synonyms = synonyms


class DictionaryTerms(BaseModel):
    """
    专有词汇列表
    """
    term_item: List[TermItem]
    page_count: int

    def __init__(self, term_item: List[TermItem], page_count: int) -> None:
        self.term_item = [TermItem.from_dict(ti) for ti in term_item]
        self.page_count = page_count


class DictionaryTerm(BaseModel):
    """
    创建/更新专有词汇
    """
    term_item: TermItem

    def __init__(self, term_item: TermItem) -> None:
        self.term_item = TermItem.from_dict(term_item)


class Value(BaseModel):
    """
    实体值
    """
    synonyms: List[str]
    standard_value: str

    def __init__(self, synonyms: List[str], standard_value: str) -> None:
        self.synonyms = synonyms
        self.standard_value = standard_value


class IntentEntityValue(Value):
    """
    意图实体值
    """
    pass


class SystemEntityValue(BaseModel):
    """
    预设实体
    """
    description: str

    def __init__(self, description: str) -> None:
        self.description = description


class RegexEntityValue(BaseModel):
    """
    正则实体
    """
    regex: str

    def __init__(self, regex: str) -> None:
        self.regex = regex


class EnumEntityValue(BaseModel):
    """
    枚举实体值
    """
    values: List[Value]

    def __init__(self, values: List[Value]) -> None:
        self.values = [Value.from_dict(value) for value in values]


class EntityValue(BaseModel):
    """
    实体值
    """
    value: dict

    def __init__(self, **kwargs) -> None:
        if "intent_entity_value" in kwargs:
            self.text = IntentEntityValue.from_dict(kwargs.get("intent_entity_value"))
        elif "system_entity_value" in kwargs:
            self.image = SystemEntityValue.from_dict(kwargs.get("system_entity_value"))
        elif "regex_entity_value" in kwargs:
            self.custom = RegexEntityValue.from_dict(kwargs.get("regex_entity_value"))
        elif "enumeration_entity_value" in kwargs:
            self.video = EnumEntityValue.from_dict(kwargs.get("enumeration_entity_value"))
        else:
            for k, v in kwargs.items():
                setattr(self, k, v)


class EntityDetail(BaseModel):
    """
    实体详情
    """
    type: str
    id: int
    value: EntityValue
    name: str

    def __init__(self, type: str, id: int, value: Value, name: str) -> None:
        self.type = type
        self.id = id
        self.value = EntityValue.from_dict(value)
        self.name = name


class DictionaryEntity(BaseModel):
    """
    查询一个实体详情
    """
    entity: EntityDetail

    def __init__(self, entity: EntityDetail) -> None:
        self.entity = EntityDetail.from_dict(entity)


class EnumEntity(BaseModel):
    """
    枚举实体详情
    """
    values: List[Value]
    id: int
    name: str

    def __init__(self, values: List[Value], id: int, name: str) -> None:
        self.values = [Value.from_dict(value) for value in values]
        self.id = id
        self.name = name


class CreateEnumEntity(BaseModel):
    """
    创建枚举实体
    """
    enum_entity: EnumEntity

    def __init__(self, enum_entity: EnumEntity) -> None:
        self.enum_entity = EnumEntity.from_dict(enum_entity)


class CreateEnumEntityValue(BaseModel):
    """
    创建枚举实体值
    """
    enum_entity: EnumEntity

    def __init__(self, enum_entity: EnumEntity) -> None:
        self.enum_entity = EnumEntity.from_dict(enum_entity)


class IntentEntity(BaseModel):
    """
    意图实体
    """
    id: int
    value: Value
    name: str

    def __init__(self, id: int, value: Value, name: str) -> None:
        self.id = id
        self.value = Value.from_dict(value)
        self.name = name


class CreateIntentEntity(BaseModel):
    """
    创建意图实体
    """
    intent_entity: IntentEntity

    def __init__(self, intent_entity: IntentEntity) -> None:
        self.intent_entity = IntentEntity.from_dict(intent_entity)


class CreateIntentEntityValue(BaseModel):
    """
    创建意图实体值相似说法
    """
    intent_entity: IntentEntity

    def __init__(self, intent_entity: IntentEntity) -> None:
        self.intent_entity = IntentEntity.from_dict(intent_entity)



"""
七、满意度收集
每次回复时让用户添加满意度评价，之后一次性获取统计值。

对接步骤：

1. 调用获取机器人回复接口（v2/msg/bot-response），获取返回字段中的 bot 和knowledge_id；

2. 调用同步发给用户的消息接口（v2/msg/sync），传入步骤1中的 bot，同时获取返回字段中的 msg_id；

3. 调用添加用户满意度评价接口（v2/qa/satisfaction/create），传入步骤1中获得的 knowledge_id 和步骤2中获得的 msg_id。

4. 调用查询问答满意度评价统计列表（知识点粒度，日报）（v2/stats/qa/satisfaction/daily/knowledge/list），
传入开始时间和结束时间（开始时间和结束时间相距不能超过30天）并获得该时间段内的满意度统计值。
"""
import os
import time

from wulaisdk.client import WulaiClient


pubkey = os.getenv("PUBKEY", "")
secret = os.getenv("SECRET", "")

client = WulaiClient(pubkey, secret, debug=False)

# 1. 获取机器人回复
user_id = "test_user"
msg_body = {
    "text": {
        "content": "你好"
    }
}
resp = client.get_bot_response(user_id, msg_body)
bot = resp.suggested_response[0].bot.to_dict()
msg_body_response = resp.suggested_response[0].response[0].msg_body.to_dict()

# 2. 同步用户消息给吾来
msg_ts = str(int(time.time() * 1000))
resp_sync = client.sync_message(user_id, msg_body_response, msg_ts, bot=bot.to_dict())

# msg_id
msg_id = resp_sync.msg_id
# knowledge_id
knowledge_id = bot.qa.knowledge_id
# 满意度评价
satisfaction = "THUMB_UP"

# 3. 添加用户满意度评价
resp = client.create_qa_satisfaction(msg_id, user_id, {"knowledge_id": str(knowledge_id)}, satisfaction)

page = 1
page_size = 50
start_date = "20190930"
end_date = "20191030"
# 4. 调用查询问答满意度评价统计列表（知识点粒度，日报）
resp_stats = client.stats_qa_satisfaction_daily_knowledges(start_date, end_date, page, page_size)



"""
用户类api测试
"""
import os
import pytest

from wulaisdk.client import WulaiClient
from wulaisdk.exceptions import ClientException

pubkey = os.getenv("PUBKEY", "")
secret = os.getenv("SECRET", "")
log_dir_path = os.path.dirname(os.path.dirname(os.path.abspath(__file__)))


#############################################################
#                          用户类
#############################################################
# createUser test
@pytest.mark.parametrize('debug,user_id,avatar_url,nickname,expected', [
    (True, "shierlou", "", "测试用户-shierlou", {}),
    (True, "shierlou", "", "", {}),
    (True, "test1", "", "", {}),
])
def test_create_user_normal_1(debug, user_id, avatar_url, nickname, expected):
    client = WulaiClient(pubkey, secret, debug=debug)
    resp = client.create_user(user_id, avatar_url, nickname)
    assert resp == expected


@pytest.mark.parametrize('debug,user_id,expected', [
    (True, "shierlou", {}),
    (True, "shierlou", {}),
])
def test_create_user_normal_2(debug, user_id, expected):
    client = WulaiClient(pubkey, secret, debug=debug)
    resp = client.create_user(user_id)
    assert resp == expected


@pytest.mark.parametrize('debug,user_id', [
    (True, ""),
    (False, 123),
])
def test_create_user_error(debug, user_id):
    client = WulaiClient(pubkey, secret, debug=debug)
    with pytest.raises(ClientException) as excinfo:
        client.create_user(user_id)
        assert excinfo.error_code == "SDK_INVALID_PARAMS"


# 获取用户属性列表
@pytest.mark.parametrize('filter, page, page_size', [
    (None, 1, 50),
    (None, 2, 30),
    ({"use_in_user_attribute_group": True}, 1, 30),
    ({"use_in_user_attribute_group": False}, 1, 30)
])
def test_user_attributes(filter, page, page_size):
    client = WulaiClient(pubkey, secret, debug=True)
    resp = client.user_attributes(page, page_size, filter)
    assert resp


# 查询用户属性值
@pytest.mark.parametrize('user_id', [
    "shierlou"
])
def test_user_user_attribute(user_id):
    client = WulaiClient(pubkey, secret, debug=True)
    resp = client.user_user_attribute(user_id)
    assert resp


# 给用户添加属性值
@pytest.mark.parametrize('user_attribute_user_attribute_value, user_id', [
    ([{"user_attribute": {"id": "101627"}, "user_attribute_value": {"name": "微博"}}], "shierlou"),
    ([{"user_attribute": {"id": "101629"}, "user_attribute_value": {"name": "老用户"}}], "shierlou"),
    ([
        {"user_attribute": {"id": "101627"}, "user_attribute_value": {"name": "微信"}},
        {"user_attribute": {"id": "101629"}, "user_attribute_value": {"name": "新用户"}}
    ], "shierlou"),
])
def test_create_user_attribute(user_attribute_user_attribute_value, user_id):
    client = WulaiClient(pubkey, secret, debug=True)
    resp = client.create_user_attribute(user_attribute_user_attribute_value, user_id)
    assert 1


# createUserUserAttribute test
# 注：接口功能同上create_user_attribute，不小心创建重复
# @pytest.mark.smoke
@pytest.mark.parametrize('user_id,user_attribute_user_attribute_value,expected', [
    ("shierlou", [
        {"user_attribute": {"id": "101634"}, "user_attribute_value": {"name": "man"}}
    ], {}),
])
def test_create_user_user_attribute_normal(user_id, user_attribute_user_attribute_value, expected):
    client = WulaiClient(pubkey, secret, debug=True)
    resp = client.create_user_user_attribute(user_id, user_attribute_user_attribute_value)
    assert resp == expected


# 查询用户信息
@pytest.mark.parametrize('user_id', [
    "shierlou"
])
def test_get_user_normal(user_id):
    client = WulaiClient(pubkey, secret, debug=True)
    resp = client.get_user(user_id)
    assert resp.nickname
    assert resp.avatar_url


@pytest.mark.parametrize('user_id', [
    "shierlou111"
])
def test_get_user_error(user_id):
    client = WulaiClient(pubkey, secret, debug=True)
    with pytest.raises(ClientException) as excinfo:
        client.get_user(user_id)
        assert excinfo.error_code == "SDK_INVALID_PARAMS"


# 更新用户信息
@pytest.mark.parametrize('user_id, avatar_url, nickname', [
    ("test1", "https://www.baidu.com/img/bd_logo1.png", "测试用户1"),
    ("test1", None, None)
])
def test_update_user(user_id, avatar_url, nickname):
    client = WulaiClient(pubkey, secret, debug=True)
    client.update_user(user_id, avatar_url, nickname)
    resp = client.get_user(user_id)
    if avatar_url:
        assert resp.avatar_url == avatar_url
        assert resp.nickname == nickname
    else:
        assert resp.avatar_url
        assert resp.nickname



"""
配置类api测试
"""
import os
import pytest

from wulaisdk.client import WulaiClient


pubkey = os.getenv("PUBKEY", "")
secret = os.getenv("SECRET", "")
log_dir_path = os.path.dirname(os.path.dirname(os.path.abspath(__file__)))


@pytest.mark.smoke
@pytest.mark.parametrize('config', [
    {
        "bot_config": {
            "qa_bot_config": {"similar_response_enabled": True, "auto_response_threshold": 0.76},
            "task_bot_config": {"auto_response_threshold": 0.79}
        },
        "default_response_config": {"enabled": True},
        "customer_service_config": {"enabled": False}
    }
])
def test_update_config(config):
    client = WulaiClient(pubkey, secret, debug=True)
    resp = client.update_config(config)
    assert resp.app_config.bot_config.qa_bot_config.auto_response_threshold
    assert resp.to_dict()



"""
知识点类
1. 创建知识点
2. 更新知识点
3. 删除知识点
4. 查询知识点列表
5. 查询知识点分类列表
6. 创建相似问
7. 更新相似问
8. 删除相似问
9. 查询相似问列表
10. 创建属性组
11. 更新属性组
12.查询属性组及属性列表
13. 创建属性组回复
14. 更新属性组回复
15. 查询属性组回复列表
16. 删除属性组回复
"""
from typing import List

from wulaisdk.response import BaseModel
from wulaisdk.response.msg_body import MsgBody
from wulaisdk.response.category_user import UserAttributeValue


class Knowledge(BaseModel):
    """
    知识点详情
    """
    status: bool
    update_time: str
    maintained_by_user_attribute_group: bool
    standard_question: str
    create_time: str
    respond_all: bool
    id: str

    def __init__(self, status: bool, update_time: str, maintained_by_user_attribute_group: bool,
                 standard_question: str, create_time: str, respond_all: bool, id: str) -> None:
        self.status = status
        self.update_time = update_time
        self.maintained_by_user_attribute_group = maintained_by_user_attribute_group
        self.standard_question = standard_question
        self.create_time = create_time
        self.respond_all = respond_all
        self.id = id


class KnowledgeTagKnowledge(BaseModel):
    """
    知识点
    """
    knowledge: Knowledge
    knowledge_tag_id: str

    def __init__(self, knowledge: Knowledge, knowledge_tag_id: str) -> None:
        self.knowledge = Knowledge.from_dict(knowledge)
        self.knowledge_tag_id = knowledge_tag_id


class KnowledgeCreate(BaseModel):
    """
    创建知识点
    """
    knowledge_tag_knowledge: KnowledgeTagKnowledge

    def __init__(self, knowledge_tag_knowledge: KnowledgeTagKnowledge) -> None:
        self.knowledge_tag_knowledge = KnowledgeTagKnowledge.from_dict(knowledge_tag_knowledge)


class KnowledgeUpdate(BaseModel):
    """
    更新知识点
    """
    knowledge: Knowledge

    def __init__(self, knowledge: Knowledge) -> None:
        self.knowledge = Knowledge.from_dict(knowledge)


class KnowledgeTag(BaseModel):
    """
    知识点分类
    """
    parent_knowledge_tag_id: str
    id: str
    name: str

    def __init__(self, parent_knowledge_tag_id: str, id: str, name: str) -> None:
        self.parent_knowledge_tag_id = parent_knowledge_tag_id
        self.id = id
        self.name = name


class SimilarQuestion(BaseModel):
    """
    相似问详情
    """
    knowledge_id: str
    question: str
    id: str

    def __init__(self, knowledge_id: str, question: str, id: str) -> None:
        self.knowledge_id = knowledge_id
        self.question = question
        self.id = id


class KnowledgeItem(BaseModel):
    """
    知识点
    """
    knowledge_tag: KnowledgeTag
    similar_questions: List[SimilarQuestion]
    knowledge: Knowledge

    def __init__(self, knowledge_tag: KnowledgeTag, similar_questions: List[SimilarQuestion], knowledge: Knowledge) -> None:
        self.knowledge_tag = KnowledgeTag.from_dict(knowledge_tag)
        self.similar_questions = [SimilarQuestion.from_dict(sq) for sq in similar_questions]
        self.knowledge = Knowledge.from_dict(knowledge)


class KnowledgeItems(BaseModel):
    """
    查询知识点列表
    """
    page_count: int
    knowledge_items: List[KnowledgeItem]

    def __init__(self, page_count: int, knowledge_items: List[KnowledgeItem]) -> None:
        self.page_count = page_count
        self.knowledge_items = [KnowledgeItem.from_dict(ki) for ki in knowledge_items]


class KnowledgeTags(BaseModel):
    """
    查询知识点分类列表
    """
    knowledge_tags: List[KnowledgeTag]
    page_count: int

    def __init__(self, knowledge_tags: List[KnowledgeTag], page_count: int) -> None:
        self.knowledge_tags = [KnowledgeTag.from_dict(kt) for kt in knowledge_tags]
        self.page_count = page_count


class KnowledgeTagCreate(BaseModel):
    """
    创建知识点分类
    """
    knowledge_tag: KnowledgeTag

    def __init__(self, knowledge_tag: KnowledgeTag) -> None:
        self.knowledge_tag = KnowledgeTag.from_dict(knowledge_tag)


class KnowledgeTagUpdate(BaseModel):
    """
    更新知识点分类
    """
    knowledge_tag: KnowledgeTag

    def __init__(self, knowledge_tag: KnowledgeTag) -> None:
        self.knowledge_tag = KnowledgeTag.from_dict(knowledge_tag)


# 相似问
class SimilarQuestionCreate(BaseModel):
    """
    创建相似问
    """
    similar_question: SimilarQuestion

    def __init__(self, similar_question: SimilarQuestion) -> None:
        self.similar_question = SimilarQuestion.from_dict(similar_question)


class SimilarQuestionUpdate(BaseModel):
    """
    更新相似问
    """
    similar_question: SimilarQuestion

    def __init__(self, similar_question: SimilarQuestion) -> None:
        self.similar_question = SimilarQuestion.from_dict(similar_question)


class SimilarQuestions(BaseModel):
    """
    查询相似问列表
    """
    similar_questions: List[SimilarQuestion]
    page_count: int

    def __init__(self, similar_questions: List[SimilarQuestion], page_count: int) -> None:
        self.similar_questions = [SimilarQuestion.from_dict(sq) for sq in similar_questions]
        self.page_count = page_count


# 属性组
class UserAttribute(BaseModel):
    """
    用户属性
    """
    value_type: str
    use_in_user_attribute_group: bool
    type: str
    id: str
    name: str

    def __init__(self, value_type: str, use_in_user_attribute_group: bool, type: str, id: str, name: str) -> None:
        self.value_type = value_type
        self.use_in_user_attribute_group = use_in_user_attribute_group
        self.type = type
        self.id = id
        self.name = name


class UserAttributeGroup(BaseModel):
    """
    用户属性组详情
    """
    id: str
    name: str

    def __init__(self, id: str, name: str) -> None:
        self.id = id
        self.name = name


class UserAttributeUserAttributeValue(BaseModel):
    """
    属性
    """
    user_attribute: UserAttribute
    user_attribute_value: UserAttributeValue

    def __init__(self, user_attribute: UserAttribute, user_attribute_value: UserAttributeValue) -> None:
        self.user_attribute = UserAttribute.from_dict(user_attribute)
        self.user_attribute_value = UserAttributeValue.from_dict(user_attribute_value)


class UserAttributeGroupItem(BaseModel):
    """
    用户属性组及属性
    """
    user_attribute_user_attribute_values: List[UserAttributeUserAttributeValue]
    user_attribute_group: UserAttributeGroup

    def __init__(self, user_attribute_user_attribute_values: List[UserAttributeUserAttributeValue],
                 user_attribute_group: UserAttribute) -> None:
        self.user_attribute_user_attribute_values = [
            UserAttributeUserAttributeValue.from_dict(uauav) for uauav in user_attribute_user_attribute_values
        ]
        self.user_attribute_group = UserAttributeGroup.from_dict(user_attribute_group)


class CreateUserAttributeGroup(BaseModel):
    """
    创建属性组
    """
    user_attribute_group_item: UserAttributeGroupItem

    def __init__(self, user_attribute_group_item: UserAttributeGroupItem) -> None:
        self.user_attribute_group_item = UserAttributeGroupItem.from_dict(user_attribute_group_item)


class UpdateUserAttributeGroup(BaseModel):
    """
    更新属性组
    """
    user_attribute_group_item: UserAttributeGroupItem

    def __init__(self, user_attribute_group_item: UserAttributeGroupItem) -> None:
        self.user_attribute_group_item = UserAttributeGroupItem.from_dict(user_attribute_group_item)


class UpdateUserAttributeGroupItems(BaseModel):
    """
    查询属性组及属性列表
    """
    page_count: int
    user_attribute_group_items: List[UserAttributeGroupItem]

    def __init__(self, page_count: int, user_attribute_group_items: List[UserAttributeGroupItem]) -> None:
        self.page_count = page_count
        self.user_attribute_group_items = [
            UserAttributeGroupItem.from_dict(uagi) for uagi in user_attribute_group_items
        ]


class Answer(BaseModel):
    """
    回复
    """
    knowledge_id: str
    msg_body: MsgBody
    id: str

    def __init__(self, knowledge_id: str, msg_body: MsgBody, id: str) -> None:
        self.knowledge_id = knowledge_id
        self.msg_body = MsgBody.from_dict(msg_body)
        self.id = id


class UserAttributeGroupAnswer(BaseModel):
    """
    属性组回复
    """
    answer: Answer
    user_attribute_group_id: str

    def __init__(self, answer: Answer, user_attribute_group_id: str) -> None:
        self.answer = Answer.from_dict(answer)
        self.user_attribute_group_id = user_attribute_group_id


class CreateUserAttributeGroupAnswer(BaseModel):
    """
    创建属性组回复
    """
    user_attribute_group_answer: UserAttributeGroupAnswer

    def __init__(self, user_attribute_group_answer: UserAttributeGroupAnswer) -> None:
        self.user_attribute_group_answer = UserAttributeGroupAnswer.from_dict(user_attribute_group_answer)


class UpdateUserAttributeGroupAnswer(BaseModel):
    """
    更新属性组回复
    """
    user_attribute_group_answer: UserAttributeGroupAnswer

    def __init__(self, user_attribute_group_answer: UserAttributeGroupAnswer) -> None:
        self.user_attribute_group_answer = UserAttributeGroupAnswer.from_dict(user_attribute_group_answer)


class UserAttributeGroupAnswers(BaseModel):
    """
    查询属性组回复列表
    """
    user_attribute_group_answers: List[UserAttributeGroupAnswer]
    page_count: int

    def __init__(self, user_attribute_group_answers: List[UserAttributeGroupAnswer], page_count: int) -> None:
        self.user_attribute_group_answers = [
            UserAttributeGroupAnswer.from_dict(uaga) for uaga in user_attribute_group_answers
        ]
        self.page_count = page_count


class KnowledgeRelatedItem(BaseModel):
    """
    知识点（包含属性组信息）
    """
    knowledge_tag: KnowledgeTag
    similar_questions: List[SimilarQuestion]
    user_attribute_group_answers: List[UserAttributeGroupAnswer]
    knowledge: Knowledge

    def __init__(self, knowledge_tag: KnowledgeTag, similar_questions: List[SimilarQuestion],
                 user_attribute_group_answers: List[UserAttributeGroupAnswer], knowledge: Knowledge) -> None:
        self.knowledge_tag = KnowledgeTag.from_dict(knowledge_tag)
        self.similar_questions = [SimilarQuestion.from_dict(similar_question) for similar_question in similar_questions]
        self.user_attribute_group_answers = [
            UserAttributeGroupAnswer.from_dict(uaga) for uaga in user_attribute_group_answers
        ]
        self.knowledge = Knowledge.from_dict(knowledge)


class KnowledgeBatchCreate(BaseModel):
    """
    批量添加知识点列表
    """
    knowledge_related_items: List[KnowledgeRelatedItem]

    def __init__(self, knowledge_related_items: List[KnowledgeRelatedItem]) -> None:
        self.knowledge_related_items = [KnowledgeRelatedItem.from_dict(kri) for kri in knowledge_related_items]



"""
用户类
1. 创建用户
2. 查询用户信息 √
3. 更新用户信息
4. 获取用户属性列表 √
5. 查询用户的属性值 √
6. 给用户添加属性值
"""
from typing import List

from wulaisdk.response import BaseModel


class UserAttributeValue(BaseModel):
    """
    用户属性值
    """
    id: str
    name: str

    def __init__(self, id: str, name: str) -> None:
        self.id = id
        self.name = name


class UserAttributeTotal(BaseModel):
    """
    用户属性
    """
    name: str
    lifespan: int
    value_type: str
    use_in_user_attribute_group: bool
    type: str
    id: str

    def __init__(self, name: str, lifespan: int, value_type: str, use_in_user_attribute_group: bool, type: str,
                 id: str) -> None:
        self.name = name
        self.lifespan = lifespan
        self.value_type = value_type
        self.use_in_user_attribute_group = use_in_user_attribute_group
        self.type = type
        self.id = id


class UserAttributeTotalUserAttributeValue(BaseModel):
    """
    属性
    """
    user_attribute: UserAttributeTotal
    user_attribute_value: UserAttributeValue

    def __init__(self, user_attribute: UserAttributeTotal, user_attribute_value: UserAttributeValue) -> None:
        self.user_attribute = UserAttributeTotal.from_dict(user_attribute)
        self.user_attribute_value = UserAttributeValue.from_dict(user_attribute_value)


class UserAttributeTotalUserAttributeValues(BaseModel):
    """
    属性列表
    """
    user_attribute: UserAttributeTotal
    user_attribute_value: List[UserAttributeValue]

    def __init__(self, user_attribute: UserAttributeTotal, user_attribute_value: List[UserAttributeValue]) -> None:
        self.user_attribute = UserAttributeTotal.from_dict(user_attribute)
        self.user_attribute_value = [UserAttributeValue.from_dict(uav) for uav in user_attribute_value]


class UserUserAttribute(BaseModel):
    """
    用户属性值
    """
    user_attribute_user_attribute_values: List[UserAttributeTotalUserAttributeValue]

    def __init__(self, user_attribute_user_attribute_values: List[UserAttributeTotalUserAttributeValue]) -> None:
        self.user_attribute_user_attribute_values = [
            UserAttributeTotalUserAttributeValue.from_dict(uauav) for uauav in user_attribute_user_attribute_values
        ]


class UserAttributes(BaseModel):
    """
    用户属性列表
    """
    page_count: int
    user_attribute_user_attribute_values: List[UserAttributeTotalUserAttributeValues]

    def __init__(self, page_count: int,
                 user_attribute_user_attribute_values: List[UserAttributeTotalUserAttributeValues]) -> None:
        self.page_count = page_count
        self.user_attribute_user_attribute_values = [
            UserAttributeTotalUserAttributeValues.from_dict(uauav) for uauav in user_attribute_user_attribute_values
        ]


class GetUser(BaseModel):
    """
    查询用户信息
    """
    avatar_url: str
    nickname: str

    def __init__(self, avatar_url: str, nickname: str) -> None:
        self.avatar_url = avatar_url
        self.nickname = nickname



"""
知识点类api测试
"""
import os
import time
import pytest
import copy
import sys

from wulaisdk.client import WulaiClient

pubkey = os.getenv("PUBKEY", "")
secret = os.getenv("SECRET", "")
log_dir_path = os.path.dirname(os.path.dirname(os.path.abspath(__file__)))


def find_value_from_dict_array(li, search_key, value_key, reference):
    """
    在[{}, {}, ...]中找到合适的值
    :param li:
    :param search_key: 需要查询的key
    :param value_key: 目标值所在key
    :param reference: 对比值
    :return:
    """
    for di in li:
        if di[search_key] == reference:
            return di[value_key]
    return None


# knowledge test
# create/update/delete knowledge tags
@pytest.mark.parametrize('knowledge_tag_create, knowledge_tag_update, knowledge_tag_child', [
    [{
        "id": "1325698",  # 指定无效，但不会报错
        "name": "测试知识点分类创建"
    }, {"name": "测试知识点分类更新"}, {"name": "测试知识点分类创建-子"}]
])
def test_knowledge_tags_new(knowledge_tag_create, knowledge_tag_update, knowledge_tag_child):
    client = WulaiClient(pubkey, secret, debug=True)

    knowledge_tag_c = client.create_knowledge_tag(knowledge_tag_create)
    knowledge_tag_id = knowledge_tag_c.knowledge_tag.id

    assert knowledge_tag_id
    assert knowledge_tag_c.knowledge_tag.name
    assert knowledge_tag_c.knowledge_tag.parent_knowledge_tag_id

    # 创建子分类
    knowledge_tag_child["parent_knowledge_tag_id"] = knowledge_tag_id
    client.create_knowledge_tag(knowledge_tag_child)

    knowledge_tag_update["id"] = knowledge_tag_id
    knowledge_tag_u = client.update_knowledge_tag(knowledge_tag_update)

    assert knowledge_tag_u.knowledge_tag.id
    assert knowledge_tag_u.knowledge_tag.name
    assert knowledge_tag_u.knowledge_tag.parent_knowledge_tag_id

    client.delete_knowledge_tag(knowledge_tag_id)


def knowledge_c_timeupdate(k):
    new = copy.deepcopy(k)
    new["knowledge"]["standard_question"] += f"_{time.strftime('%Y-%m-%d/%H:%M:%S',time.localtime())}"
    return new


def knowledge_u_timeupdate(knowledge_c):
    new = copy.deepcopy(knowledge_c)
    knowledge_u = new["knowledge"]
    knowledge_u["standard_question"] += f"-改_{time.strftime('%Y-%m-%d/%H:%M:%S',time.localtime())}"
    return knowledge_u


knowledge_c_1 = {
    "knowledge": {
        "status": True,
        "standard_question": "sdk测试什么才是标准问",
        "respond_all": True,
        "maintained_by_user_attribute_group": True
    },
    "knowledge_tag_id": ""
}
knowledge_c_2 = {
    "knowledge": {
        "status": True,
        "standard_question": "sdk测试如何退换货",
        "respond_all": True,
        "maintained_by_user_attribute_group": True,
        "id": None
    },
    "knowledge_tag_id": ""
}


# normal
# knowledge_tags / knowledge_items / create_knowledge / update_knowledge / similar-questions
@pytest.mark.parametrize('page,page_size,knowledge_c,knowledge_u', [
    (1, 50, knowledge_c_timeupdate(knowledge_c_1), knowledge_u_timeupdate(knowledge_c_1)),
    (1, 50, knowledge_c_timeupdate(knowledge_c_2), knowledge_u_timeupdate(knowledge_c_2)),
])
def test_knowledge_tags(page, page_size, knowledge_c, knowledge_u):
    client = WulaiClient(pubkey, secret, debug=True)
    # tags
    #    -- without parent_k_tag_id
    resp_parent_tags = client.knowledge_tags(page, page_size)
    parent_k_tag_id = resp_parent_tags.knowledge_tags[0].id

    #    -- with parent_k_tag_id
    resp_second_tags = client.knowledge_tags(page, page_size, parent_k_tag_id)
    knowledge_tags = resp_second_tags.knowledge_tags
    knowledge_tags_di = [r.to_dict() for r in knowledge_tags]
    target_id = find_value_from_dict_array(knowledge_tags_di, "name", "id", "测试_yt")

    # create
    knowledge_c["knowledge_tag_id"] = target_id
    resp_knowledge_c = client.create_knowledge(knowledge_c)
    knowledge_id = resp_knowledge_c.knowledge_tag_knowledge.knowledge.id

    # update
    knowledge_u["id"] = knowledge_id
    resp_knowledge_u = client.update_knowledge(knowledge_u)

    # items
    # 过滤查询
    resp_knowledges = client.knowledge_items(page, page_size=50, knowledge_filter={"knowledge_tag_id": target_id})
    assert resp_knowledges.to_dict()
    # 无条件查询
    resp = client.knowledge_items(page, page_size=10)

    assert resp_knowledge_u.knowledge.id == knowledge_id
    assert resp.page_count

    # similar-question
    # create
    similar_question_c = {
        "knowledge_id": knowledge_id,
        "question": "我能在你左边画条龙嘛？"
    }
    resp_similar_c = client.create_similar_question(similar_question_c)

    # update
    similar_question_id = resp_similar_c.similar_question.id
    similar_question_u = {
        "knowledge_id": knowledge_id,
        "question": "我能在你左边画条龙嘛？-改",
        "id": similar_question_id
    }
    resp_similar_u = client.update_similar_question(similar_question_u)

    # list
    resp_similar_list = client.similar_questions(page, page_size, knowledge_id, timeout=10)
    assert resp_similar_u.similar_question.to_dict() in [r.to_dict() for r in resp_similar_list.similar_questions]

    # delete
    resp = client.delete_similar_question(similar_question_id)
    assert isinstance(resp, dict)

    # delete knowledge
    resp = client.delete_knowledge(knowledge_id)
    assert isinstance(resp, dict)


# user-attribute test
# 区分3.6和3.7的测试内容
if sys.version_info.minor == 7:
    source_name = "微信"
else:
    source_name = "微博"
user_attribute_group_item_1 = {
    "user_attribute_user_attribute_value": [
        {
            "user_attribute": {"id": "101627"},
            "user_attribute_value": {"name": source_name}
        },
        {
            "user_attribute": {"id": "101629"},
            "user_attribute_value": {"name": "新用户"}
        }
    ],
    "user_attribute_group": {"name": f"{source_name}新用户"}
}
user_attribute_group_item_update_1 = {
    "user_attribute_user_attribute_value": [
        {
            "user_attribute": {"id": "101627"},
            "user_attribute_value": {"name": source_name}
        },
        {
            "user_attribute": {"id": "101629"},
            "user_attribute_value": {"name": "老用户"}
        }
    ],
    "user_attribute_group": {
        "id": "",
        "name": f"{source_name}老用户"
    }
}
user_attribute_group_answer_1 = {
    "answer": {
        "knowledge_id": "1048247",
        "msg_body": {"text": {"content": f"这是测试属性组的答案--{source_name}老用户"}},
    },
    "user_attribute_group_id": ""
}
user_attribute_group_answer_update_1 = {
    "answer": {
        "knowledge_id": "1048247",
        "msg_body": {"text": {"content": f"这是测试属性组的答案--{source_name}老用户"}},
        "id": ""
    },
    "user_attribute_group_id": ""
}


@pytest.mark.parametrize('user_attribute_group_item,user_attribute_group_item_update,user_attribute_group_answer,'
                         'user_attribute_group_answer_update,page,page_size', [
                             (user_attribute_group_item_1, user_attribute_group_item_update_1,
                              user_attribute_group_answer_1,
                              user_attribute_group_answer_update_1, 1, 50),
                         ])
def test_user_attribute(user_attribute_group_item, user_attribute_group_item_update, user_attribute_group_answer,
                        user_attribute_group_answer_update, page, page_size):
    # 属性组测试知识点分类id：125537，知识点id： 1048247
    client = WulaiClient(pubkey, secret, debug=True)
    # 创建属性组
    # todo：每一次测试后需要去项目里手动删除这个属性组
    resp_cuagi = client.create_user_attribute_group_items(user_attribute_group_item)
    # 更新属性组
    #   属性组id
    user_attribute_group_id = resp_cuagi.user_attribute_group_item.user_attribute_group.id

    user_attribute_group_item_update["user_attribute_group"]["id"] = user_attribute_group_id

    resp_uuagi = client.update_user_attribute_group_items(user_attribute_group_item_update)

    #   属性组名称
    user_attribute_group_name = resp_uuagi.user_attribute_group_item.user_attribute_group.name

    # 查询属性组及属性列表
    resp_uagi = client.user_attribute_group_items(page, page_size, timeout=10)
    user_attribute_group_ids = [uagi.user_attribute_group.id for uagi in resp_uagi.user_attribute_group_items]
    user_attribute_group_names = [uagi.user_attribute_group.name for uagi in resp_uagi.user_attribute_group_items]
    assert user_attribute_group_id in user_attribute_group_ids
    assert user_attribute_group_name in user_attribute_group_names

    # 创建属性组回复
    user_attribute_group_answer["user_attribute_group_id"] = user_attribute_group_id

    resp_cuaga = client.create_user_attribute_group_answer(user_attribute_group_answer)
    #   属性组答案id
    user_attribute_group_answer_id = resp_cuaga.user_attribute_group_answer.answer.id
    # 更新属性组回复
    user_attribute_group_answer_update["answer"]["id"] = user_attribute_group_answer_id
    user_attribute_group_answer_update["user_attribute_group_id"] = user_attribute_group_id

    resp_uuaga = client.update_user_attribute_group_answer(user_attribute_group_answer_update)
    assert resp_uuaga.user_attribute_group_answer.answer.id == user_attribute_group_answer_id
    # 查询属性组回复列表
    resp_answers = client.user_attribute_group_answers(page, page_size, timeout=10)
    resp_answers_2 = client.user_attribute_group_answers(page, page_size, {"knowledge_id": "1048247"}, timeout=10)
    assert isinstance(resp_answers.user_attribute_group_answers, list)
    assert isinstance(resp_answers_2.user_attribute_group_answers, list)

    # 删除属性组回复
    resp = client.delete_user_attribute_group_answer(user_attribute_group_answer_id)


knowledge_items = [
    {
        "knowledge_tag": {
            "parent_knowledge_tag_id": "57247",
            "id": "117858",
            "name": "测试_yt"
        },
        "similar_questions": [
            {
                # "knowledge_id": "2321501",
                "question": "相似问111",
                # "id": "20835353"
            }
        ],
        "user_attribute_group_answers": [
            {
                "answer": {
                    # "knowledge_id": "2321501",
                    "msg_body": {
                        "text": {
                            "content": "测试文本消息"
                        }
                    }
                },
                "user_attribute_group_id": "0"
            }
        ],
        "knowledge": {
            "status": True,
            "update_time": str(int(time.time())),
            "maintained_by_user_attribute_group": False,
            "standard_question": "批量添加1",
            "respond_all": True,
            # "id": "2321501"
        }
    }
]


# 批量添加知识点
# batch_create_knowledge
@pytest.mark.parametrize('knowledge_items', [
    knowledge_items
])
def test_knowledge_tags(knowledge_items):
    client = WulaiClient(pubkey, secret, debug=True)

    resp = client.batch_create_knowledge(knowledge_items)

    assert resp.to_dict()
    knowledge_id = resp.knowledge_related_items[0].knowledge.id
    knowledge_tag_id = resp.knowledge_related_items[0].knowledge_tag.id
    assert knowledge_tag_id

    client.delete_knowledge(knowledge_id)



import os
import pytest
import logging

from wulaisdk.client import WulaiClient
from wulaisdk.request import CommonRequest
from wulaisdk.exceptions import ClientException

pubkey = os.getenv("PUBKEY", "")
secret = os.getenv("SECRET", "")
log_dir_path = os.path.dirname(os.path.dirname(os.path.abspath(__file__)))


@pytest.mark.parametrize('debug,action,params,expected', [
    (False, '/user/create',
     {
         "avatar_url": "",
         "user_id": "shierlou",
         "nickname": "测试用户-shierlou"
     },
     {}),
    (True, '/user/create',
     {
         "avatar_url": "",
         "user_id": "shierlou",
         "nickname": "测试用户-shierlou"
     },
     {}),
])
def test_common_request(debug, action, params, expected):
    opts = {
        "method": "POST",
        "timeout": 3,
        "retry": 0,
    }
    client = WulaiClient(pubkey, secret, debug=debug)
    request = CommonRequest(action, params, opts)
    resp = client.process_common_request(request)
    assert resp == expected


# test invalid http method
@pytest.mark.parametrize('debug,action,params,opts', [
    (False, '/user/create',
     {
         "avatar_url": "",
         "user_id": "shierlou",
         "nickname": "测试用户-shierlou"
     },
     {
         "method": "PUT",
         "timeout": 3,
         "retry": 0,
     }),
    (True, '/user/create',
     {
         "avatar_url": "",
         "user_id": "shierlou",
         "nickname": "测试用户-shierlou"
     },
     {
         "method": "GET",
         "timeout": 3,
         "retry": 0,
     }),
])
def test_http_method(debug, action, params, opts):
    client = WulaiClient(pubkey, secret, debug=debug)
    with pytest.raises(ClientException) as excinfo:
        request = CommonRequest(action, params, opts)
        client.process_common_request(request)
        assert excinfo.error_msg == "SDK_METHOD_NOT_ALLOW"


# test api_version
@pytest.mark.parametrize('debug,user_id,avatar_url,nickname', [
    (True, "shierlou", "", "测试用户-shierlou"),
    (True, "shierlou", "", ""),
])
def test_api_version(debug, user_id, avatar_url, nickname):
    with pytest.raises(ClientException) as excinfo:
        client = WulaiClient(pubkey, secret, debug=debug, api_version="3")
        client.create_user(user_id, avatar_url, nickname)
        assert excinfo.error_code == "SDK_INVALID_API_VERSION"


# test log FileHandler
@pytest.mark.parametrize('handler', [
    logging.FileHandler(log_dir_path + '/logs/wulaisdk.log')
])
def test_log_handler(handler):
    client = WulaiClient(pubkey, secret, debug=True)
    client.add_logger_handler(handler)
    client.create_user("shierlou")


# test invalid credential
@pytest.mark.parametrize('debug,user_id,avatar_url,nickname', [
    (True, "shierlou", "", "测试用户-shierlou"),
    (True, "shierlou", "", ""),
])
def test_credential(debug, user_id, avatar_url, nickname):
    client = WulaiClient("asEGUJtCyLkHvTmm8vE0bXLe5ebGs2PP00df19808441f2cfff", secret, debug=debug)
    with pytest.raises(ClientException) as excinfo:
        client.create_user(user_id, avatar_url, nickname)
        assert excinfo.error_code == "SDK_INVALID_CREDENTIAL"





"""
词库管理类api测试
"""
import os
import pytest
import random

from wulaisdk.client import WulaiClient


pubkey = os.getenv("PUBKEY", "")
secret = os.getenv("SECRET", "")
log_dir_path = os.path.dirname(os.path.dirname(os.path.abspath(__file__)))


#############################################################
#                       自然语言处理类
#############################################################
# 实体抽取测试
@pytest.mark.parametrize('query', [
    '灵刻车型有悦翔V5，江南TT和逸动',
    '我的ip地址是180.167.137.10',
])
def test_entities_extract(query):
    client = WulaiClient(pubkey, secret, debug=True)

    resp = client.entities_extract(query)
    assert resp.to_dict()
    for entity in resp.entities:
        if hasattr(entity.entity, "system_entity"):
            assert entity.entity.system_entity.standard_value
        elif hasattr(entity.entity, "enumeration_entity"):
            assert entity.entity.enumeration_entity.standard_value
        elif hasattr(entity.entity, "regex_entity"):
            assert entity.entity.regex_entity.name
        else:
            # error
            assert False


# 分词&词性标注
@pytest.mark.parametrize('query', [
    '灵刻车型有悦翔V5，江南TT和逸动',
    '我的ip地址是180.167.137.10',
])
def test_tokenize(query):
    client = WulaiClient(pubkey, secret, debug=True)

    resp = client.tokenize(query)
    assert resp.tokens[0].text


# 导入待聚类语料
@pytest.mark.parametrize('queries', [
    ['今天天气怎么样', '今天是晴天', '今天吃什么', '你好', '你真棒'],
    ['教练我想打篮球', '武磊牛逼', '湖人输了', '你看nba吗', '今天天气不错'],
])
def test_mining_upload(queries):
    client = WulaiClient(pubkey, secret, debug=True)

    resp = client.mining_upload(queries)
    assert resp.succeeded_count or resp.duplicated_count


# 发起聚类
def test_mining_execute():
    client = WulaiClient(pubkey, secret, debug=True)

    resp = client.mining_execute()
    assert resp.status


# 获取聚类结果列表
def test_mining_result():
    client = WulaiClient(pubkey, secret, debug=True)

    resp = client.mining_result(1, 50)
    # 聚类中
    if resp.status == "MINING_STATUS_IN_PROGRESS":
        assert 1
    # 聚类结束
    else:
        if resp.page_count:
            sentence_ids = []
            for cluster in resp.clusters:
                for sentence in cluster.sentences:
                    sentence_ids.append(sentence.id)
            del_id = random.choice(sentence_ids)
            # 删除聚类结果
            client.delete_mining_sentence(del_id)


# 清空待聚类语料
def test_mining_empty():
    client = WulaiClient(pubkey, secret, debug=True)

    client.mining_empty()



"""
任务类
1. 查询场景列表
2. 创建场景
3. 更新场景
4. 删除场景
5. 查询意图列表
6. 创建意图
7. 更新意图
8. 删除意图
9. 查询触发器列表
10. 创建触发器
11. 更新触发器
12. 删除触发器
13. 查询词槽列表
14. 创建词槽
15. 更新词槽
16. 查询词槽
17. 删除词槽
18. 查询词槽数据来源
19. 创建词槽数据来源
20. 删除词槽数据来源
21. 查询消息发送单元
22. 创建消息发送单元
23. 更新消息发送单元
24. 查询询问填槽单元
25. 创建询问填槽单元
26. 更新询问填槽单元
27. 创建单元内回复
28.  更新单元内回复
29. 删除单元内回复
30. 查询意图终点单元
31. 创建意图终点单元
32. 更新意图终点单元
33. 查询单元列表
34. 创建单元关系
35. 删除单元关系
36. 删除单元
37. 查询任务待审核消息列表
38. 删除任务待审核消息
39. 更新意图状态
"""
from typing import List
from wulaisdk.response import BaseModel
from wulaisdk.response.msg_body import MsgBody


class Scene(BaseModel):
    """
    场景
    """
    description: str
    intent_switch_mode: str
    id: int
    smart_slot_filling_threshold: int
    name: str

    def __init__(self, description: str, intent_switch_mode: str, id: int,
                 smart_slot_filling_threshold: int, name: str) -> None:
        self.description = description
        self.intent_switch_mode = intent_switch_mode
        self.id = id
        self.smart_slot_filling_threshold = smart_slot_filling_threshold
        self.name = name


class Intent(BaseModel):
    """
    意图
    """
    scene_id: int
    status: bool
    lifespan_mins: int
    id: int
    name: str

    def __init__(self, scene_id: int, status: bool, lifespan_mins: int, id: int, name: str) -> None:
        self.scene_id = scene_id
        self.status = status
        self.lifespan_mins = lifespan_mins
        self.id = id
        self.name = name


class IntentTrigger(BaseModel):
    """
    触发器
    """
    text: str
    intent_id: int
    type: str
    id: int

    def __init__(self, text: str, intent_id: int, type: str, id: int) -> None:
        self.text = text
        self.intent_id = intent_id
        self.type = type
        self.id = id


class SlotSimple(BaseModel):
    """
    词槽(简版)
    """
    id: int
    name: str

    def __init__(self, id: int, name: str) -> None:
        self.id = id
        self.name = name


class Slot(BaseModel):
    """
    词槽
    """
    scene_id: int
    query_slot_filling: bool
    id: int
    name: str

    def __init__(self, scene_id: int, query_slot_filling: bool, id: int, name: str) -> None:
        self.scene_id = scene_id
        self.query_slot_filling = query_slot_filling
        self.id = id
        self.name = name


class DataSource(BaseModel):
    """
    词槽数据来源
    """
    entity_id: int
    slot_id: int
    id: int

    def __init__(self, entity_id: int, slot_id: int, id: int) -> None:
        self.entity_id = entity_id
        self.slot_id = slot_id
        self.id = id


class RecommendIntent(BaseModel):
    """
    推荐意图
    """
    intent_id: int
    score: int
    intent_name: str

    def __init__(self, intent_id: int, score: int, intent_name: str) -> None:
        self.intent_id = intent_id
        self.score = score
        self.intent_name = intent_name


class QueryItem(BaseModel):
    """
    待审核问题
    """
    content: str
    id: int
    recommend_intent: RecommendIntent

    def __init__(self, content: str, id: int, recommend_intent: RecommendIntent) -> None:
        self.content = content
        self.id = id
        self.recommend_intent = RecommendIntent.from_dict(recommend_intent)


# api start
class Scenes(BaseModel):
    """
    查询场景列表
    """
    scenes: List[Scene]

    def __init__(self, scenes: List[Scene]) -> None:
        self.scenes = [Scene.from_dict(scene) for scene in scenes]


class CreateScene(BaseModel):
    """
    创建场景
    """
    scene: Scene

    def __init__(self, scene: Scene) -> None:
        self.scene = Scene.from_dict(scene)


class UpdateScene(BaseModel):
    """
    更新场景
    """
    scene: Scene

    def __init__(self, scene: Scene) -> None:
        self.scene = Scene.from_dict(scene)


class Intents(BaseModel):
    """
    查询意图列表
    """
    intents: List[Intent]

    def __init__(self, intents: List[Intent]) -> None:
        self.intents = [Intent.from_dict(intent) for intent in intents]


class CreateIntent(BaseModel):
    """
    创建意图
    """
    intent: Intent

    def __init__(self, intent: Intent) -> None:
        self.intent = Intent.from_dict(intent)


class UpdateIntent(BaseModel):
    """
    更新意图
    """
    intent: Intent

    def __init__(self, intent: Intent) -> None:
        self.intent = Intent.from_dict(intent)


class IntentTriggers(BaseModel):
    """
    触发器列表
    """
    intent_triggers: List[IntentTrigger]

    def __init__(self, intent_triggers: List[IntentTrigger]) -> None:
        self.intent_triggers = [IntentTrigger.from_dict(intent_trigger) for intent_trigger in intent_triggers]


class CreateIntentTrigger(BaseModel):
    """
    创建触发器
    """
    intent_trigger: IntentTrigger

    def __init__(self, intent_trigger: IntentTrigger) -> None:
        self.intent_trigger = IntentTrigger.from_dict(intent_trigger)


class UpdateIntentTrigger(BaseModel):
    """
    更新触发器
    """
    intent_trigger: IntentTrigger

    def __init__(self, intent_trigger: IntentTrigger) -> None:
        self.intent_trigger = IntentTrigger.from_dict(intent_trigger)


class Slots(BaseModel):
    """
    词槽列表
    """
    slots: List[SlotSimple]

    def __init__(self, slots: List[SlotSimple]) -> None:
        self.slots = [SlotSimple.from_dict(slot) for slot in slots]


class CreateSlot(BaseModel):
    """
    创建词槽
    """
    slot: Slot

    def __init__(self, slot: Slot) -> None:
        self.slot = Slot.from_dict(slot)


class UpdateSlot(BaseModel):
    """
    更新词槽
    """
    slot: Slot

    def __init__(self, slot: Slot) -> None:
        self.slot = Slot.from_dict(slot)


class GetSlot(BaseModel):
    """
    词槽详情
    """
    slot: Slot

    def __init__(self, slot: Slot) -> None:
        self.slot = Slot.from_dict(slot)


class SlotDataSource(BaseModel):
    """
    查询词槽数据来源
    """
    data_sources: List[DataSource]

    def __init__(self, data_sources: List[DataSource]) -> None:
        self.data_sources = [DataSource.from_dict(data_source) for data_source in data_sources]


class CreateSlotDataSource(BaseModel):
    """
    创建词槽数据来源
    """
    data_source: DataSource

    def __init__(self, data_source: DataSource) -> None:
        self.data_source = DataSource.from_dict(data_source)


class Response(BaseModel):
    """
    单元内回复
    """
    response: MsgBody
    id: int
    block_id: int

    def __init__(self, response: MsgBody, id: int, block_id: int) -> None:
        self.response = MsgBody.from_dict(response)
        self.id = id
        self.block_id = block_id


class Last(BaseModel):
    """
    意图终点跳转上个意图
    """

    def __init__(self, **kwargs) -> None:
        for k, v in kwargs.items():
            self.k = v


class End(BaseModel):
    """
    意图终点单元跳转指定意图
    """

    def __init__(self, **kwargs) -> None:
        for k, v in kwargs.items():
            self.k = v


class Specified(BaseModel):
    """
    意图终点单元跳转指定意图
    """
    id: int

    def __init__(self, id: int) -> None:
        self.id = id


class Action(BaseModel):
    """
    结束单元跳转方式 (指定意图 / 上个意图 / 不跳转)
    """

    def __init__(self, **kwargs) -> None:
        if "last" in kwargs:
            self.last = Last.from_dict(kwargs.get("last"))
        elif "end" in kwargs:
            self.end = End.from_dict(kwargs.get("end"))
        elif "specified" in kwargs:
            self.specified = Specified.from_dict(kwargs.get("specified"))
        else:
            for k, v in kwargs.items():
                setattr(self, k, v)


class EndBlock(BaseModel):
    """
    意图终点单元
    """
    action: Action
    slot_memorizing: bool
    intent_id: int
    id: int
    name: str

    def __init__(self, action: Action, slot_memorizing: bool, intent_id: int, id: int, name: str) -> None:
        self.action = Action.from_dict(action)
        self.slot_memorizing = slot_memorizing
        self.intent_id = intent_id
        self.id = id
        self.name = name


class Block(BaseModel):
    """
    单元
    """
    type: str
    id: int
    name: str

    def __init__(self, type: str, id: int, name: str) -> None:
        self.type = type
        self.id = id
        self.name = name


# 单元
class Default(BaseModel):
    """
    单元跳转条件 默认
    """

    def __init__(self, **kwargs) -> None:
        for k, v in kwargs.items():
            self.k = v


class InEntity(BaseModel):
    """
    单元跳转条件 属于(实体)
    """
    id: int

    def __init__(self, id: int) -> None:
        self.id = id


class NotInEntity(BaseModel):
    """
    单元跳转条件 不属于(实体)
    """
    id: int

    def __init__(self, id: int) -> None:
        self.id = id


class EqualTo(BaseModel):
    """
    单元跳转条件 等于
    """
    value: str

    def __init__(self, value: str) -> None:
        self.value = value


class NotEqualTo(BaseModel):
    """
    单元跳转条件 不等于
    """
    value: str

    def __init__(self, value: str) -> None:
        self.value = value


class LessThan(BaseModel):
    """
    单元跳转条件 小于
    """
    value: float

    def __init__(self, value: float) -> None:
        self.value = value


class LessThanOrEqualTo(BaseModel):
    """
    单元跳转条件 小于等于
    """
    value: float

    def __init__(self, value: float) -> None:
        self.value = value


class GreaterThan(BaseModel):
    """
    单元跳转条件 大于
    """
    value: float

    def __init__(self, value: float) -> None:
        self.value = value


class GreaterThanOrEqualTo(BaseModel):
    """
    单元跳转条件 大于等于
    """
    value: float

    def __init__(self, value: float) -> None:
        self.value = value


class MatchRegex(BaseModel):
    """
    单元跳转条件 符合正则
    """
    regex: str

    def __init__(self, regex: str) -> None:
        self.regex = regex


class DismatchRegex(BaseModel):
    """
    单元跳转条件 不符合正则
    """
    regex: str

    def __init__(self, regex: str) -> None:
        self.regex = regex


class Include(BaseModel):
    """
    单元跳转条件 包含
    """
    value: str

    def __init__(self, value: str) -> None:
        self.value = value


class Exclude(BaseModel):
    """
    单元跳转条件 不包含
    """
    value: str

    def __init__(self, value: str) -> None:
        self.value = value


class Condition(BaseModel):
    """
    单元跳转条件(默认 / 大于 / 大于等于 / 小于 / 小于等于 / 等于 / 不等于 / 包含 / 不包含 / 属于实体 / 不属于实体 / 符合正则 / 不符合正则)
    """

    def __init__(self, **kwargs) -> None:
        if "default" in kwargs:
            self.default = Default.from_dict(kwargs.get("default"))
        elif "in_entity" in kwargs:
            self.in_entity = InEntity.from_dict(kwargs.get("in_entity"))
        elif "not_in_entity" in kwargs:
            self.not_in_entity = NotInEntity.from_dict(kwargs.get("not_in_entity"))
        elif "equal_to" in kwargs:
            self.equal_to = EqualTo.from_dict(kwargs.get("equal_to"))
        elif "not_equal_to" in kwargs:
            self.not_equal_to = NotEqualTo.from_dict(kwargs.get("not_equal_to"))
        elif "greater_than" in kwargs:
            self.greater_than = GreaterThan.from_dict(kwargs.get("greater_than"))
        elif "greater_than_or_equal_to" in kwargs:
            self.greater_than_or_equal_to = GreaterThanOrEqualTo.from_dict(kwargs.get("greater_than_or_equal_to"))
        elif "less_than" in kwargs:
            self.less_than = LessThan.from_dict(kwargs.get("less_than"))
        elif "less_than_or_equal_to" in kwargs:
            self.less_than_or_equal_to = LessThanOrEqualTo.from_dict(kwargs.get("less_than_or_equal_to"))
        elif "match_regex" in kwargs:
            self.match_regex = MatchRegex.from_dict(kwargs.get("match_regex"))
        elif "match_regex" in kwargs:
            self.match_regex = MatchRegex.from_dict(kwargs.get("match_regex"))
        elif "dismatch_regex" in kwargs:
            self.dismatch_regex = DismatchRegex.from_dict(kwargs.get("dismatch_regex"))
        elif "include" in kwargs:
            self.include = Include.from_dict(kwargs.get("include"))
        elif "exclude" in kwargs:
            self.exclude = Exclude.from_dict(kwargs.get("exclude"))
        else:
            for k, v in kwargs.items():
                setattr(self, k, v)


class Connection(BaseModel):
    """
    单元关系
    """
    from_block_id: int
    to_block_id: int
    condition: Condition

    def __init__(self, from_block_id: int, to_block_id: int, condition: Condition) -> None:
        self.from_block_id = from_block_id
        self.to_block_id = to_block_id
        self.condition = Condition.from_dict(condition)


class Relation(BaseModel):
    """
    单元关系
    """
    connection: Connection
    intent_id: int
    id: int

    def __init__(self, connection: Connection, intent_id: int, id: int) -> None:
        self.connection = Connection.from_dict(connection)
        self.intent_id = intent_id
        self.id = id


class InformBlock(BaseModel):
    """
    消息发送单元
    """
    responses: List[MsgBody]
    connection: Connection
    mode: str
    intent_id: int
    id: int
    name: str

    def __init__(self, responses: List[MsgBody], connection: Connection, mode: str, intent_id: int, id: int, name: str) -> None:
        self.responses = [MsgBody.from_dict(response) for response in responses]
        self.connection = Connection.from_dict(connection)
        self.mode = mode
        self.intent_id = intent_id
        self.id = id
        self.name = name


class RequestBlock(BaseModel):
    """
    询问填槽单元
    """
    name: str
    default_slot_value: str
    slot_filling_when_asked: bool
    connections: List[Connection]
    slot_id: int
    mode: str
    request_count: int
    intent_id: int
    id: int
    responses: List[MsgBody]

    def __init__(
            self, name: str, default_slot_value: str, slot_filling_when_asked: bool,
            connections: List[Connection], slot_id: int, mode: str, request_count: int,
            intent_id: int, id: int, responses: List[MsgBody]) -> None:
        self.name = name
        self.default_slot_value = default_slot_value
        self.slot_filling_when_asked = slot_filling_when_asked
        self.connections = [Connection.from_dict(connection) for connection in connections]
        self.slot_id = slot_id
        self.mode = mode
        self.request_count = request_count
        self.intent_id = intent_id
        self.id = id
        self.responses = [MsgBody.from_dict(response) for response in responses]


class GetInformBlock(BaseModel):
    """
    查询消息发送单元
    """
    block: InformBlock

    def __init__(self, block: InformBlock) -> None:
        self.block = InformBlock.from_dict(block)


class CreateInformBlock(BaseModel):
    """
    创建消息发送单元
    """
    block: InformBlock

    def __init__(self, block: InformBlock) -> None:
        self.block = InformBlock.from_dict(block)


class UpdateInformBlock(BaseModel):
    """
    更新消息发送单元
    """
    block: InformBlock

    def __init__(self, block: InformBlock) -> None:
        self.block = InformBlock.from_dict(block)


class GetRequestBlock(BaseModel):
    """
    查询询问填槽单元
    """
    block: RequestBlock

    def __init__(self, block: RequestBlock) -> None:
        self.block = RequestBlock.from_dict(block)


class CreateRequestBlock(BaseModel):
    """
    创建询问填槽单元
    """
    block: RequestBlock

    def __init__(self, block: RequestBlock) -> None:
        self.block = RequestBlock.from_dict(block)


class UpdateRequestBlock(BaseModel):
    """
    更新询问填槽单元
    """
    block: RequestBlock

    def __init__(self, block: RequestBlock) -> None:
        self.block = RequestBlock.from_dict(block)


class CreateResponse(BaseModel):
    """
    创建单元内回复
    """
    response: Response

    def __init__(self, response: Response) -> None:
        self.response = Response.from_dict(response)


class UpdateResponse(BaseModel):
    """
    更新单元内回复
    """
    response: Response

    def __init__(self, response: Response) -> None:
        self.response = Response.from_dict(response)


class GetEndBlock(BaseModel):
    """
    查询意图终点单元
    """
    block: EndBlock

    def __init__(self, block: EndBlock) -> None:
        self.block = EndBlock.from_dict(block)


class CreateEndBlock(BaseModel):
    """
    创建意图终点单元
    """
    block: EndBlock

    def __init__(self, block: EndBlock) -> None:
        self.block = EndBlock.from_dict(block)


class UpdateEndBlock(BaseModel):
    """
    更新意图终点单元
    """
    block: EndBlock

    def __init__(self, block: EndBlock) -> None:
        self.block = EndBlock.from_dict(block)


class Blocks(BaseModel):
    """
    查询单元列表
    """
    blocks: List[Block]

    def __init__(self, blocks: List[Block]) -> None:
        self.blocks = [Block.from_dict(block) for block in blocks]


class CreateBlockRelation(BaseModel):
    """
    创建单元关系
    """
    relation: Relation

    def __init__(self, relation: Relation) -> None:
        self.relation = Relation.from_dict(relation)


class IntentTriggerLearning(BaseModel):
    """
    待审核问题列表
    """
    query_items: List[QueryItem]

    def __init__(self, query_items: List[QueryItem]) -> None:
        self.query_items = [QueryItem.from_dict(query_item) for query_item in query_items]


class UpdateIntentStatus(BaseModel):
    """
    更新意图状态
    """
    status: bool
    first_block_id: int
    intent_id: int
    update_time: str

    def __init__(self, status: bool, first_block_id: int, intent_id: int, update_time: str) -> None:
        self.status = status
        self.first_block_id = first_block_id
        self.intent_id = intent_id
        self.update_time = update_time



"""
对话类api测试
"""
import os
import time
import pytest

from wulaisdk.client import WulaiClient
from wulaisdk.response.category_talk import BotResponse
from wulaisdk.response.msg_body import MsgBody


pubkey = os.getenv("PUBKEY", "")
secret = os.getenv("SECRET", "")
log_dir_path = os.path.dirname(os.path.dirname(os.path.abspath(__file__)))


#############################################################
#                          对话类
#############################################################
# getBotResponse test
@pytest.mark.parametrize('user_id,msg_body,extra', [
    ("shierlou", {"text": {"content": "你好"}}, "this is extra string")
])
def test_get_bot_response_normal(user_id, msg_body, extra):
    client = WulaiClient(pubkey, secret, debug=True)
    resp = client.get_bot_response(user_id, msg_body, extra)
    assert resp.suggested_response[0].response[0].msg_body.text.content == '你好，有什么可以为你服务的吗？(*╹▽╹*)'


# getBotResponse for image test
@pytest.mark.parametrize('user_id,msg_body', [
    ("shierlou", {"text": {"content": "图片如何回复"}})
])
def test_get_bot_response_image(user_id, msg_body):
    client = WulaiClient(pubkey, secret, debug=True)
    resp = client.get_bot_response(user_id, msg_body)
    assert isinstance(resp.suggested_response[0].response[0].msg_body.image.resource_url, str)


# getKeyWordBotResponse test
@pytest.mark.parametrize('user_id,msg_body,expected', [
    ("shierlou", {"text": {"content": "关键词测试"}}, "哈哈哈，这是关键词😃")
])
def test_get_keyword_bot_response_normal(user_id, msg_body, expected):
    client = WulaiClient(pubkey, secret, debug=True)
    resp = client.get_keyword_bot_response(user_id, msg_body)
    assert resp.to_dict()["keyword_suggested_response"][0]["response"][0]["msg_body"]["text"]["content"] == expected
    assert resp.keyword_suggested_response[0].response[0].msg_body.text.content == expected


# getQABotResponse test
@pytest.mark.parametrize('user_id,msg_body,extra,expected', [
    ("shierlou", {"text": {"content": "吃了吗"}}, "this is extra string", "服务你第一，吃饭第二，请问有什么可以帮助你的吗？")
])
def test_get_qa_bot_response_normal(user_id, msg_body, extra, expected):
    client = WulaiClient(pubkey, secret, debug=True)
    resp = client.get_qa_bot_response(user_id, msg_body, extra)
    assert resp.to_dict()["qa_suggested_response"][0]["response"][0]["msg_body"]["text"]["content"] == expected
    assert resp.qa_suggested_response[0].response[0].msg_body.text.content == expected


# getQABotResponse for image test
@pytest.mark.parametrize('user_id,msg_body,extra', [
    ("shierlou", {"text": {"content": "图片如何回复"}}, "this is extra string")
])
def test_get_qa_bot_response_image(user_id, msg_body, extra):
    client = WulaiClient(pubkey, secret, debug=True)
    resp = client.get_qa_bot_response(user_id, msg_body, extra)
    assert isinstance(resp.qa_suggested_response[0].response[0].msg_body.image.resource_url, str)


# getTaskBotResponse test
@pytest.mark.parametrize('user_id,msg_body,extra', [
    ("shierlou", {"text": {"content": "我想查尺码"}}, "this is extra string")
])
def test_get_task_bot_response_normal(user_id, msg_body, extra):
    client = WulaiClient(pubkey, secret, debug=True)
    resp = client.get_task_bot_response(user_id, msg_body, extra)
    assert "text" in resp.to_dict()["task_suggested_response"][0]["response"][0]["msg_body"].keys()
    assert isinstance(resp.task_suggested_response[0].response[0].msg_body.text.content, str)


# syncMessage test
@pytest.mark.parametrize('user_id,msg_body,msg_ts', [
    ("shierlou", {"text": {"content": "测试message_sync"}}, str(int(time.time() * 1000)))
])
def test_sync_message(user_id, msg_body, msg_ts):
    client = WulaiClient(pubkey, secret, debug=True)
    resp = client.sync_message(user_id, msg_body, msg_ts)
    assert "msg_id" in resp.to_dict()
    assert isinstance(resp.msg_id, str)


# receiveMessage test
@pytest.mark.parametrize('user_id,msg_body', [
    ("shierlou", {"text": {"content": "测试receive_message"}})
])
def test_receive_message(user_id, msg_body):
    client = WulaiClient(pubkey, secret, debug=True)
    resp = client.receive_message(user_id, msg_body)
    assert "msg_id" in resp.to_dict()
    assert isinstance(resp.msg_id, str)


# getMessageHistory test
@pytest.mark.parametrize('user_id,num', [
    ("shierlou", 50)
])
def test_message_history(user_id, num):
    client = WulaiClient(pubkey, secret, debug=True)
    resp = client.get_message_history(user_id, num)
    assert resp.to_dict()
    assert isinstance(resp.msg[0].user_info.nickname, str)
    assert isinstance(resp.msg[0].msg_body.to_dict(), dict)
    msg_body = resp.msg[0].msg_body
    if hasattr(msg_body, "text"):
        assert isinstance(msg_body.text.content, str)
    elif hasattr(msg_body, "image"):
        assert isinstance(msg_body.image.resource_url, str)


# bot-response wrapper test
@pytest.mark.parametrize('body', [
    {"msg_id": "123", "test": "test", "is_dispatch": False, "suggested_response": [], "extra": ""}
])
def test_err_response(body):
    assert BotResponse.from_dict(body)


# msg_body wrapper test
@pytest.mark.parametrize('body', [
    {"text": {"content": "hhhh"}},
    {"image": {"resource_url": "hhhh"}},
    {"test": {"test": "test"}},
])
def test_err_response(body):
    assert MsgBody.from_dict(body)


# 给用户发消息
@pytest.mark.parametrize('user_id, msg_body, quick_reply, similar_response, extra', [
    ("shierlou", {"text": {"content": "测试给用户发消息"}}, None, None, None),
    ("shierlou", {"text": {"content": "测试给用户发消息2"}}, ["快捷回复1", "快捷回复2"], {
        "source": "QA_BOT",
        "detail": {
            "qa": {
                "knowledge_id": "",
                "standard_question": "",
                "question": "",
                "is_none_intention": False
            }
        }
    }, ""),
])
def test_send_message(user_id, msg_body, quick_reply, similar_response, extra):
    client = WulaiClient(pubkey, secret, debug=True)
    resp = client.send_message(user_id, msg_body, quick_reply, similar_response, extra)
    assert resp.msg_id


# 获取用户输入联想
@pytest.mark.parametrize('user_id, query', [
    ("shierlou", "你好"),
])
def test_send_message(user_id, query):
    client = WulaiClient(pubkey, secret, debug=True)
    resp = client.get_user_suggestion(user_id, query)
    user_suggestiont = resp.user_suggestions[0]
    assert user_suggestiont.suggestion


#############################################################
#                   reset task bot
#############################################################
@pytest.mark.parametrize('user_id,msg_body,extra', [
    ("shierlou", {"text": {"content": "/restart"}}, "this is extra string")
])
def test_reset_task(user_id, msg_body, extra):
    client = WulaiClient(pubkey, secret, debug=True)
    resp = client.get_bot_response(user_id, msg_body, extra)
    assert isinstance(resp.suggested_response[0].response[0].msg_body.text.content, str)



"""
配置类
1. 更新机器人回复配置
"""
from wulaisdk.response import BaseModel


class QABotConfig(BaseModel):
    """
    问答对话配置
    """
    similar_response_enabled: bool
    auto_response_threshold: int

    def __init__(self, similar_response_enabled: bool, auto_response_threshold: int) -> None:
        self.similar_response_enabled = similar_response_enabled
        self.auto_response_threshold = auto_response_threshold


class TaskBotConfig(BaseModel):
    """
    任务对话配置
    """
    auto_response_threshold: int

    def __init__(self, auto_response_threshold: int) -> None:
        self.auto_response_threshold = auto_response_threshold


class BotConfig(BaseModel):
    """
    对话配置
    """
    qa_bot_config: QABotConfig
    task_bot_config: TaskBotConfig

    def __init__(self, qa_bot_config: QABotConfig, task_bot_config: TaskBotConfig) -> None:
        self.qa_bot_config = QABotConfig.from_dict(qa_bot_config)
        self.task_bot_config = TaskBotConfig.from_dict(task_bot_config)


class EConfig(BaseModel):
    """
    兜底回复/转人工配置
    """
    enabled: bool

    def __init__(self, enabled: bool) -> None:
        self.enabled = enabled


class AppConfig(BaseModel):
    """
    机器人配置
    """
    bot_config: BotConfig
    default_response_config: EConfig
    customer_service_config: EConfig

    def __init__(self, bot_config: BotConfig, default_response_config: EConfig, customer_service_config: EConfig) -> None:
        self.bot_config = BotConfig.from_dict(bot_config)
        self.default_response_config = EConfig.from_dict(default_response_config)
        self.customer_service_config = EConfig.from_dict(customer_service_config)


class UpdateConfig(BaseModel):
    """
    更新机器人配置
    """
    app_config: AppConfig

    def __init__(self, app_config: AppConfig) -> None:
        self.app_config = AppConfig.from_dict(app_config)
